--[[
Name: Hemlock
Revision: $Rev: 1 $
Developed by: Antiarc
Documentation: 
SVN: http://svn.wowace.com/wowace/trunk/Hemlock
Description: Minimalistic addon to automate poison buying and creation
Dependencies: AceLibrary, Dewdrop-2.0
]]--

Hemlock = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")
if not AceLibrary("Dewdrop-2.0") then error(MAJOR_VERSION .. " requires Dewdrop-2.0") end	
Hemlock:RegisterDB("HemlockDB", "HemlockDBPC")
local dewdrop = AceLibrary("Dewdrop-2.0")

--[[
	-- Item IDs - using these precludes the need to hardcode icon paths or localize item names.
	Crippling: 3775
	Instant: 6947
	Wound: 10918
	Mind-numbing: 5237
	Anesthetic: 21835
	Deadly: 2892
	
	-- I need the Deathweed to check if it's a poison vendor. Yay for locale-agnostic code!
	Deathweed: 5173
]]--


-- Hemlock will try to get info for these from the server if the cache doesn't have it.
-- Note that if the server hasn't seen it, this will result in a client disconnect.
-- Right now, this just excludes Anesthetic
local safeIDs = {6947, 5173, 3775}

-- These should be the rank 1 poison IDs - ie, without a rank suffix!
local poisonIDs = {6947, 2892, 3775, 10918, 5237, 21835, 5530}
local poisons = {}
local poisonIcons = {}

local reagentIDs = {5140}
local reagents = {}
local reagentIcons = {}

local defaults = {
	poisonRequirements = {},
	reagentRequirements = {}
}

for k,v in ipairs(poisonIDs) do
	local itemName, _, _, _, _, _, _, _, _, invTexture = GetItemInfo(v)
	if itemName then
		tinsert(poisons, itemName)
		tinsert(poisonIcons, invTexture)
		defaults.poisonRequirements[itemName] = 0
	end
end

for k,v in ipairs(reagentIDs) do
	local itemName, _, _, _, _, _, _, _, _, invTexture = GetItemInfo(v)
	if itemName then
		tinsert(reagents, itemName)
		tinsert(reagentIcons, invTexture)
		defaults.reagentRequirements[itemName] = 0
	end
end

Hemlock:RegisterDefaults("profile", defaults)
function Hemlock:Register()
	local options = {
	    type='group',
	    args = {
	    	scan = {
	    		type = "execute",
	    		func = function() Hemlock:ScanPoisons(1) end,
	    		name = "Scan Poisons",
	    		desc = "Scan poison tradeskills for usable information"
	    	}
	    }
	}
	
	for k, v in pairs(self.db.profile.poisonRequirements) do
		options.args[gsub(k, " ", "")] = {
			type = 'range',
			name = k,
			min = 0,
			max = 100,
			step = 5,
			isPercent = false,
			desc = self:L("specify") .. " " .. k .. " " .. self:L("automade"),
			get = function()
				return self.db.profile.poisonRequirements[k]
			end,
			set = function(v2)
				self.db.profile.poisonRequirements[k] = v2
			end
		}
	end

	for k, v in pairs(self.db.profile.reagentRequirements) do
		options.args[gsub(k, " ", "")] = {
			type = 'range',
			name = k,
			min = 0,
			max = 100,
			step = 5,
			isPercent = false,
			desc = self:L("specify") .. " " .. k .. " " .. self:L("autobuy"),
			get = function()
				return self.db.profile.reagentRequirements[k]
			end,
			set = function(v2)
				self.db.profile.reagentRequirements[k] = v2
			end
		}
	end

	Hemlock:RegisterChatCommand({"/hemlock"}, options)
end

function Hemlock:OnInitialize()
	self:Register()
	self.enabled = false
	self:RegisterEvent("MERCHANT_SHOW");
	self:RegisterEvent("BAG_UPDATE");

	for k,v in ipairs(safeIDs) do
		local c, tc = UnitClass("player")
		if not GetItemInfo(v) then
			self:Print(v)
			self:Print(tc)
			self:Print(UnitLevel("player"))
			if tc == "ROGUE" and UnitLevel("player") >= 20 then
				StaticPopup_Show("HEMLOCK_NOTIFY_NEED_SCAN")
				break
			end
		end
	end	
	self.inited = false
end

function Hemlock:InitFrames()
	local lastFrame = nil
	local frameCt = 0
	for k,v in ipairs(poisonIDs) do
		local itemName, _, _, _, _, _, _, _, _, invTexture = GetItemInfo(v)
		if itemName then
			local f = CreateFrame("Button", "HemlockPoisonFrame" .. k, HemlockFrame, "HemlockPoisonTemplate")
			if k == 1 then
				f:SetPoint("TOP", HemlockFrame, "TOP", 0, -5)
			else
				f:SetPoint("TOP", lastFrame, "BOTTOM", 0, -5)
			end
			f:SetNormalTexture(invTexture)
			lastFrame = f
			f.tooltipText = self:L("clicktobuy") .. "\n" .. self:L("clicktoset",itemName)
			local menu = {
				type = "group",
				args = {
					slider = {
						type = 'range',
						name = itemName,
						min = 0,
						max = 100,
						step = 5,
						isPercent = false,
						desc = self:L("specify") .. " " .. itemName .. " " .. self:L("automade"),
						get = function()
							return self.db.profile.poisonRequirements[itemName]
						end,
						set = function(v2)
							self.db.profile.poisonRequirements[itemName] = v2
							dewdrop:GetOpenedParent():SetText(
								Hemlock:GetPoisonsInInventory(itemName) .. "\n" .. Hemlock.db.profile.poisonRequirements[itemName]
							)
						end
					}
				}
			}		
			f:SetText(self:GetPoisonsInInventory(itemName) .. "\n" .. self.db.profile.poisonRequirements[itemName])
			f:SetScript("OnClick", function()
				if TradeSkillFrame and TradeSkillFrame:IsVisible() then
					Hemlock:GetNeededPoisons(itemName, f)
				else
					CastSpellByName("Poisons")
					if TradeSkillFrame and not TradeSkillFrame:IsVisible() then
						CastSpellByName("Poisons")
					end
					if TradeSkillFrame and TradeSkillFrame:IsVisible() then
						CastSpellByName("Poisons")
					end
					Hemlock:GetNeededPoisons(itemName, f)
				end
			end)
			dewdrop:Register(f, 'children', menu)
			frameCt = frameCt + 1
		end
	end
	
	for k,v in pairs(reagentIDs) do
		local itemName, _, _, _, _, _, _, _, _, invTexture = GetItemInfo(v)
		if itemName then
			local f = CreateFrame("Button", "HemlockReagentFrame" .. k, HemlockFrame, "HemlockPoisonTemplate")
			f:SetPoint("TOP", lastFrame, "BOTTOM", 0, -5)
			f:SetNormalTexture(invTexture)
			lastFrame = f
			f.tooltipText = self:L("clicktobuy") .. "\n" .. self:L("clicktoset", itemName)
			local menu = {
				type = "group",
				args = {
					slider = {
						type = 'range',
						name = itemName,
						min = 0,
						max = 100,
						step = 5,
						isPercent = false,
						desc = self:L("specify") .. " " .. itemName .. " " .. self:L("autobuy"),
						get = function()
							return self.db.profile.reagentRequirements[itemName]
						end,
						set = function(v2)
							self.db.profile.reagentRequirements[itemName] = v2
							dewdrop:GetOpenedParent():SetText(
								GetItemCount(itemName) .. "\n" .. Hemlock.db.profile.reagentRequirements[itemName]
							)
						end
					}
				}
			}		
			f:SetText(GetItemCount(itemName) .. "\n" .. self.db.profile.reagentRequirements[itemName])
			f:SetScript("OnClick", function()
				local toBuy = self.db.profile.reagentRequirements[itemName] - GetItemCount(itemName)
				if toBuy > 0 then
					Hemlock:BuyVendorItem(itemName, toBuy)
				else
					Hemlock:Print(self:L("skippingReagent", itemName, self.db.profile.reagentRequirements[itemName], GetItemCount(itemName)))
				end
			end)
			dewdrop:Register(f, 'children', menu)
			frameCt = frameCt + 1
		end	
	end
	HemlockFrame:SetHeight(37 * frameCt + 5)
end

function Hemlock:OnEnable()
	self.enabled = true
end

function Hemlock:OnDisable()
	self.enabled = false
end

function Hemlock:MERCHANT_SHOW()
	local localclass, trueclass = UnitClass("player")
	if trueclass ~= "ROGUE" or not IsUsableSpell("Poisons") then return end

	if not self.inited then
		-- self:InitItems()
		self:InitFrames()
		self.inited = true
	end	
	HemlockFrame:Hide()

	self.needPoisonCheck = true
	for i = 1, GetMerchantNumItems() do
		local link = GetMerchantItemLink(i)
		-- If this is a deathweed vendor, we'll assume he's selling poison.
		if link and strfind(link, "Hitem:5173:") then
			HemlockFrame:Show()
			Hemlock:BAG_UPDATE()
			break
		end
	end
end

function Hemlock:BAG_UPDATE(bag_id)
	if HemlockFrame:IsVisible() then
		for k,v in ipairs(poisonIDs) do
			local f = getglobal("HemlockPoisonFrame" .. k)
			if f then
				local itemName, _, _, _, _, _, _, _, _, invTexture = GetItemInfo(v)
				f:SetText(self:GetPoisonsInInventory(itemName) .. "\n" .. self.db.profile.poisonRequirements[itemName])
				f:Enable()
				f:GetNormalTexture():SetDesaturated(false)
			end
		end
		for k,v in ipairs(reagentIDs) do
			local f = getglobal("HemlockReagentFrame" .. k)
			if f then
				local itemName, _, _, _, _, _, _, _, _, invTexture = GetItemInfo(v)
				f:SetText(GetItemCount(itemName) .. "\n" .. self.db.profile.reagentRequirements[itemName])
				f:Enable()
				f:GetNormalTexture():SetDesaturated(false)
			end
		end
	end
end

function Hemlock:GetPoisonsInInventory(name)
	local rankStrings = {"XX", "XIX", "XVIII", "XVII", "XVI", "XV", "XIV", "XIII", "XII", "XI", "X", "IX", "VIII", "VII", "VI", "V", "IV", "III", "II", "I"}
	for idx, str in ipairs(rankStrings) do
		if GetItemCount(name .. " " .. str) > 0 then
			return GetItemCount(name .. " " .. str)
		end
	end
	if GetItemCount(name) > 0 then
		return GetItemCount(name)
	end
	return 0
end

function Hemlock:GetMaxPoisonRank(poisonName)
	local ranks = {}
	poisonName = gsub(poisonName, "%-", "%%%-")
	for i = 1, GetNumTradeSkills() do
		local name, type = GetTradeSkillInfo(i)
		if type ~= "header" then
			if strfind(name, poisonName, 1) then
				tinsert(ranks, {name, i})
			end
		end
	end
	
	-- 20 is way more than we need, but eh.
	local rankStrings = {"XX", "XIX", "XVIII", "XVII", "XVI", "XV", "XIV", "XIII", "XII", "XI", "X", "IX", "VIII", "VII", "VI", "V", "IV", "III", "II", "I"}
	for idx, str in ipairs(rankStrings) do
		for k, v in ipairs(ranks) do
			if v[1] == poisonName .. " " .. str then 
				return v[1], v[2]
			end
		end
	end
	
	return ranks[1][1], ranks[1][2]
end

function Hemlock:BuyVendorItem(pName, count, countTo)
	if MerchantFrame:IsVisible() then
		for i = 1, GetMerchantNumItems() do
			local name, tex, price, qty, available, usable = GetMerchantItemInfo(i)
			local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, invTexture = GetItemInfo(GetMerchantItemLink(i))
			if name == pName then
				local ct = 0
				if count == nil and countTo ~= nil then
					local pCt = GetItemCount(GetMerchantItemLink(i))
					ct = pCt - countTo
				elseif count then
					ct = math.ceil(count / qty)
				end
				while ct > 0 do
					local ctam = 0
					if ct > math.floor(itemStackCount / qty) then
						ctam = math.floor(itemStackCount / qty) 
					else
						ctam = ct
					end
					ct = ct - ctam
					BuyMerchantItem(i, ctam)
				end
				return true
			end
		end
	end
	return false
end

function Hemlock:GetNeededPoisons(name, frame)
	local v = name
	local amt = self.db.profile.poisonRequirements[name]
	local poison, skillIndex = self:GetMaxPoisonRank(v)
	local noMessage = false
	if poison then
		local count = GetItemCount(GetTradeSkillItemLink(skillIndex))
		local toMake = math.ceil((amt - count) / GetTradeSkillNumMade(skillIndex))			
		if toMake > 0 then
			for i = 1, GetTradeSkillNumReagents(skillIndex) do
				local reagentName, reagentTexture, reagentCount, playerReagentCount = GetTradeSkillReagentInfo(skillIndex, i)
				local toBuy = (reagentCount * toMake) - playerReagentCount
				if toBuy > 0 then
					frame:Disable()
					frame:GetNormalTexture():SetDesaturated(true)
					self:ScheduleEvent(function() frame:Enable(); frame:GetNormalTexture():SetDesaturated(false) end, 2.5)
					local buyResult = self:BuyVendorItem(reagentName, toBuy)
					if not buyResult then
						Hemlock:Print(self:L("unableToBuy", toBuy, reagentName))
						noMessage = true
					end
				end
			end
			for i = 1, GetTradeSkillNumReagents(skillIndex) do
				local reagentName, reagentTexture, reagentCount, playerReagentCount = GetTradeSkillReagentInfo(skillIndex, i)
				local toBuy = (reagentCount * toMake) - playerReagentCount
				if toBuy > 0 then
					local bags = {0,1,2,3,4}
					for k,v in ipairs(bags) do
						local slots = GetContainerNumSlots(v)
						for j = 1, slots do
							if not noMessage and not GetContainerItemLink(v, j) then
								self:Print(self:L("pleasepress",  name))
								return
							end
						end
					end
					break
				end
			end
			DoTradeSkill(skillIndex, toMake)
		else
			Hemlock:Print(self:L("skipping", name, amt, count))
		end
	end
end

function Hemlock:L(str, ...)
	local s = nil
	if HemlockLocalization[GetLocale()] and HemlockLocalization[GetLocale()][str] then
		s = HemlockLocalization[GetLocale()][str]
	elseif HemlockLocalization["enUS"] and HemlockLocalization["enUS"][str] then
		s = HemlockLocalization["enUS"][str]
	else
		s = "INVALID_LOCALIZATION_KEY: " .. str
	end
	for k,v in pairs({...}) do
		s = gsub(s, "$S", v, 1)
	end
	return s
end

StaticPopupDialogs["HEMLOCK_NOTIFY_NEED_SCAN"] = {
	text = "Hemlock needs to scan your poison tradeskills to gather information. Do so now?",
	button1 = TEXT(YES),
	button2 = TEXT(NO),
	timeout = 0,
	showAlert = 1,
	hideOnEscape = 1,
};

StaticPopupDialogs["HEMLOCK_NOTIFY_NEED_SCAN"].OnAccept = function()
	Hemlock:ScanPoisons(1)
end

function Hemlock:ScanPoisons(step)
	if step == 1 then
		for i=1,3 do
			CastSpellByName("Poisons")
			if TradeSkillFrame and TradeSkillFrame:IsVisible() then
				break
			end
		end
		CastSpellByName("Poisons")
		self:Print("[1/3] Reading poison information, please wait...")
		self:ScheduleEvent(function() self:ScanPoisons(2) end, 4)
	end
	
	if step == 2 or step == 3 then
		for i = 1, GetNumTradeSkills() do
			local skillName, skillType, numAvailable, isExpanded = GetTradeSkillInfo(i)
			if skillType ~= "header" then
				GameTooltip:SetTradeSkillItem(i)
				local link = GetTradeSkillItemLink(i)
			end
		end
		if step == 2 then
			self:Print("[2/3] Obtained poison list...")
		end
		self:ScheduleEvent(function() self:ScanPoisons(step + 1) end, 2)
	end
	
	if step == 4 then
		for i = 1, GetNumTradeSkills() do
			local skillName, skillType, numAvailable, isExpanded = GetTradeSkillInfo(i)
			if skillType ~= "header" then
				GameTooltip:SetTradeSkillItem(i)
				local link = GetTradeSkillItemLink(i)
				if link then
					for n = 1,2 do
						for j = 1, GetTradeSkillNumReagents(i) do
							GameTooltip:SetTradeSkillItem(i, j)
							link = GetTradeSkillReagentItemLink(i, j)
						end			
					end
				end
			end
		end
		self:Print("[3/3] Obtained reagent list. Finished!")
	end
end
